-- Greetings! Here's a script that will allow you to spawn and play with traffic in scenarios. If this looks complicated, don't worry; there will be many detailed comments so that you can try to use this stuff in your own scenarios, too! Lines that say "ESSENTIAL" are the most important lines to copy if you just want to use them for easy traffic setup.

local M = {}

local scenario = require("scenario/scenarios") -- useful for getting the scenario timer, custom finish triggers, and more

local player = {} -- general player object
local timeLimit = 200 -- scenario time limit
local waypoints = {"hd_wp1", "hd_wp2", "hd_wp3", "hd_wp4", "hd_wp5", "hd_wp6", "hd_wp7", "hd_wp8", "hd_wp9"} -- list of race waypoint names
local timeLevels = {120, 150, 190, 200} -- list of various goal times to achieve, just for fun
local text = { -- lines of text to use for scenario fail or win screens
	win = "Yay, you made it! ",
	time1 = "Wow, that was insanely speedy! You're not just a delivery driver, you're a top tier racer!",
	time2 = "Great driving, that was faster than expected. Applause for you!",
	time3 = "Enjoy the party!",
	time4 = "The timing was too close, the party was getting quite impatient!",
	crash = "Well, looks like you're stuck there! That's a fail.",
	late = "You ran out of time! What a disappointment."
}
local group = {{"pigeon", "600", "0.8 0.1 0.1 1.2"}, {"pigeon", "600", "0.18 0.36 0.09 1.3"}, {"pigeon", "600", "0.1 0.22 0.5 1.3"}, {"pigeon", "600", "0.38 0.18 0.5 1.3"}, {"pigeon", "600", "0.9 0.4 0 1.3"}, {"pigeon", "600", "0.05 0.5 0.7 1.3"}, {"pigeon", "600", "0.86 0.66 0.1 1.3"}} -- red, green, blue, purple, orange, cyan, yellow

local quantity = Engine.Platform.getCPUInfo().coresPhysical or 4 -- number of user CPU cores; default 4
quantity = clamp(quantity - 1, 3, 7) -- number of traffic vehicles to use; I set the minimum to 3 and the maximum to 7
local damageLimit = 10000 -- amount of damage that can start the player fail timer
local win = true -- allows the success message at the end of the scenario

local function onScenarioLoaded()
	core_multiSpawn.setGroup("custom", group)
	core_multiSpawn.setupVehicles(quantity, true, nil, 40) -- ESSENTIAL: This function creates a spawn group, and then spawns new vehicles ahead of the player. Arguments:
	-- quantity: The number of vehicles to spawn
	-- order: If false, shuffle the spawn group
	-- spawnMode: The method used to place the new vehicles (by default, they get placed on the road ahead)
	-- spawnGap: The distance between each spawn point, in metres
	log("A", "hectic_delivery_2", "Traffic vehicles spawned: "..quantity) -- optional message to display in the debug console; press (~)
end

local function onScenarioUIReady()
	player.obj = be:getPlayerVehicle(0) -- player vehicle object
	player.id = player.obj:getID() -- player vehicle ID
	player.failTimer = 0 -- player fail timer; can trigger after a wreck
	player.obj:queueLuaCommand("electrics.set_lightbar_signal(0)")
	
	win = true

	gameplay_traffic.activate(true) -- ESSENTIAL: This function activates the traffic mode. If the argument is set to "true", then all other vehicles are automatically used for traffic; otherwise, you can use "gameplay_traffic.insertTraffic(id)" to manually add vehicles to the traffic mode.
	gameplay_traffic.setTrafficVars({spawnIn = 3, spawnOut = 3})
	gameplay_traffic.setVehicleVars({colorMode = false})
end

local function onRaceTick(tick)
	local mainTimer = scenario.getScenario().timer -- main timer

	player.damage = map.objects[player.id].damage -- player damage
	player.vel = map.objects[player.id].vel:length() -- player velocity, in metres per second
	
	if player.damage >= damageLimit then -- if player is damaged enough
		if player.vel <= 4 then -- if the player is moving slow enough
			player.failTimer = player.failTimer + tick -- increase the fail timer
		else
			player.failTimer = 0 -- reset the fail timer
		end
	end
	
	if player.failTimer >= 4 then -- if the player is damaged and stuck for more than 4 seconds
		win = false
		scenario.finish({failed = text.crash})
	end
	
	if mainTimer >= timeLimit then -- if the time limit is reached
		win = false
		scenario.finish({failed = text.late})
	end
end

local function onRaceResult()
	if win then
		local finishTime = scenario.getScenario().timer
		local finishText = text.win
		local extraText = ""
		
		for i, v in ipairs(timeLevels) do -- loop that goes through the values in the timeLevels table
			if finishTime <= v then -- if the finish time is less than the current value in the table
				extraText = text["time"..i] -- extra text will match the time level
				break -- break and exit the loop
			end
		end
		
		scenario.finish({msg = finishText..extraText})
	end
end

M.onScenarioLoaded = onScenarioLoaded
M.onScenarioUIReady = onScenarioUIReady
M.onRaceTick = onRaceTick
M.onRaceResult = onRaceResult

return M